@@ -45,6 +45,7 @@
 #include <X11/keysym.h>
 #endif
 
+#include "gnome-settings-profile.h"
 #include "gsd-keyboard-manager.h"
 
 #include "gsd-keyboard-xkb.h"
@@ -52,6 +53,10 @@
 
 #define GSD_KEYBOARD_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_KEYBOARD_MANAGER, GsdKeyboardManagerPrivate))
 
+#ifndef HOST_NAME_MAX
+#  define HOST_NAME_MAX 255
+#endif
+
 #define GSD_KEYBOARD_KEY "/desktop/gnome/peripherals/keyboard"
 
 #define KEY_REPEAT        "/desktop/gnome/peripherals/keyboard/repeat"
@@ -67,11 +72,9 @@
 
 struct GsdKeyboardManagerPrivate
 {
-        gboolean dummy;
-};
-
-enum {
-        PROP_0,
+        gboolean have_xkb;
+        gint     xkb_event_base;
+        guint    notify;
 };
 
 static void     gsd_keyboard_manager_class_init  (GsdKeyboardManagerClass *klass);
@@ -108,6 +111,7 @@ xfree86_set_keyboard_autorepeat_rate (int delay, int rate)
         return res;
 }
 #endif /* HAVE_X11_EXTENSIONS_XF86MISC_H */
+
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
 static gboolean
 xkb_set_keyboard_autorepeat_rate (int delay, int rate)
@@ -125,11 +129,7 @@ xkb_set_keyboard_autorepeat_rate (int delay, int rate)
 static char *
 gsd_keyboard_get_hostname_key (const char *subkey)
 {
-#ifdef HOST_NAME_MAX
         char hostname[HOST_NAME_MAX + 1];
-#else
-        char hostname[256];
-#endif
 
         if (gethostname (hostname, sizeof (hostname)) == 0 &&
             strcmp (hostname, "localhost") != 0 &&
@@ -143,7 +143,7 @@ gsd_keyboard_get_hostname_key (const char *subkey)
                                    escaped,
                                    "/0/",
                                    subkey,
-                                   (char *)NULL);
+                                   NULL);
                 g_free (escaped);
                 return key;
         } else
@@ -152,26 +152,43 @@ gsd_keyboard_get_hostname_key (const char *subkey)
 
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
 
-enum {
+typedef enum {
         NUMLOCK_STATE_OFF = 0,
         NUMLOCK_STATE_ON = 1,
         NUMLOCK_STATE_UNKNOWN = 2
-};
+} NumLockState;
 
-/* something fatal has happened so that it makes no
- * sense to try to remember anything.
- * that means: no calls to the set_state functions!
- */
-static gboolean
-numlock_setup_error = FALSE;
+static void
+numlock_xkb_init (GsdKeyboardManager *manager)
+{
+        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+        gboolean have_xkb;
+        int opcode, error_base, major, minor;
 
-/* we didn't apply GConf settings yet
- * don't overwrite them with the initial state from
- * the newly started session!
- */
-static gboolean
-numlock_starting_up = TRUE;
+        gdk_error_trap_push ();
+        have_xkb = XkbQueryExtension (dpy,
+                                      &opcode,
+                                      &manager->priv->xkb_event_base,
+                                      &error_base,
+                                      &major,
+                                      &minor)
+                && XkbUseExtension (dpy, &major, &minor);
 
+        if (have_xkb) {
+                XkbSelectEventDetails (dpy,
+                                       XkbUseCoreKbd,
+                                       XkbStateNotifyMask,
+                                       XkbModifierLockMask,
+                                       XkbModifierLockMask);
+        } else {
+                g_warning ("XKB extension not available");
+        }
+
+        XSync (dpy, FALSE);
+        gdk_error_trap_pop ();
+
+        manager->priv->have_xkb = have_xkb;
+}
 
 static unsigned
 numlock_NumLock_modifier_mask (void)
@@ -181,7 +198,7 @@ numlock_NumLock_modifier_mask (void)
 }
 
 static void
-numlock_set_xkb_state (gboolean new_state)
+numlock_set_xkb_state (NumLockState new_state)
 {
         unsigned int num_mask;
         Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
@@ -196,13 +213,12 @@ numlock_gconf_state_key (void)
 {
         char *key = gsd_keyboard_get_hostname_key ("numlock_on");
         if (!key) {
-                numlock_setup_error = TRUE;
-                g_warning ("numlock: Numlock remembering disabled because your hostname is set to \"localhost\".");
+                g_warning ("NumLock remembering disabled because hostname is set to \"localhost\"");
         }
         return key;
 }
 
-static int
+static NumLockState
 numlock_get_gconf_state (GConfClient *client)
 {
         int          curr_state;
@@ -216,29 +232,28 @@ numlock_get_gconf_state (GConfClient *client)
         curr_state = gconf_client_get_bool (client, key, &err);
         if (err) {
                 curr_state = NUMLOCK_STATE_UNKNOWN;
+                g_error_free (err);
         }
 
-        g_clear_error (&err);
         g_free (key);
         return curr_state;
 }
 
 static void
 numlock_set_gconf_state (GConfClient *client,
-                         gboolean     new_state)
+                         NumLockState new_state)
 {
         char *key;
 
         if (new_state != NUMLOCK_STATE_ON && new_state != NUMLOCK_STATE_OFF) {
                 return;
         }
-        key = numlock_gconf_state_key ();
-        if (!key) {
-                return;
-        }
 
-        gconf_client_set_bool (client, key, new_state, NULL);
-        g_free (key);
+        key = numlock_gconf_state_key ();
+        if (key) {
+                gconf_client_set_bool (client, key, new_state, NULL);
+                g_free (key);
+        }
 }
 
 static GdkFilterReturn
@@ -246,7 +261,7 @@ numlock_xkb_callback (GdkXEvent *xev_,
                       GdkEvent *gdkev_,
                       gpointer xkb_event_code)
 {
-        XEvent *xev = (XEvent *)xev_;
+        XEvent *xev = (XEvent *) xev_;
 
         if (xev->type == GPOINTER_TO_INT (xkb_event_code)) {
                 XkbEvent *xkbev = (XkbEvent *)xev;
@@ -255,48 +270,23 @@ numlock_xkb_callback (GdkXEvent *xev_,
                         unsigned num_mask = numlock_NumLock_modifier_mask ();
                         unsigned locked_mods = xkbev->state.locked_mods;
                         int numlock_state = !! (num_mask & locked_mods);
-
-                        if (!numlock_starting_up && !numlock_setup_error) {
-                                GConfClient *client;
-                                client = gconf_client_get_default ();
-                                numlock_set_gconf_state (client, numlock_state);
-                                g_object_unref (client);
-                        }
+                        GConfClient *client = gconf_client_get_default ();
+                        numlock_set_gconf_state (client, numlock_state);
+                        g_object_unref (client);
                 }
         }
         return GDK_FILTER_CONTINUE;
 }
 
 static void
-numlock_install_xkb_callback (void)
+numlock_install_xkb_callback (GsdKeyboardManager *manager)
 {
-        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        int      op_code = 0;
-        int      xkb_event_code = 0;
-        int      error_code = 0;
-        int      major = XkbMajorVersion;
-        int      minor = XkbMinorVersion;
-        int      have_xkb = XkbQueryExtension (dpy,
-                                          &op_code,
-                                          &xkb_event_code,
-                                          &error_code,
-                                          &major,
-                                          &minor);
-        if (have_xkb != True) {
-                numlock_setup_error = TRUE;
-                g_warning ("numlock: XkbQueryExtension returned an error");
+        if (!manager->priv->have_xkb)
                 return;
-        }
-
-        XkbSelectEventDetails (dpy,
-                               XkbUseCoreKbd,
-                               XkbStateNotifyMask,
-                               XkbModifierLockMask,
-                               XkbModifierLockMask);
 
         gdk_window_add_filter (NULL,
                                numlock_xkb_callback,
-                               GINT_TO_POINTER (xkb_event_code));
+                               GINT_TO_POINTER (manager->priv->xkb_event_base));
 }
 
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
@@ -346,7 +336,7 @@ apply_settings (GConfClient        *client,
 
                 XAutoRepeatOn (GDK_DISPLAY ());
                 /* Use XKB in preference */
-#if defined (HAVE_X11_EXTENSIONS_XKB_H)
+#ifdef HAVE_X11_EXTENSIONS_XKB_H
                 rate_set = xkb_set_keyboard_autorepeat_rate (delay, rate);
 #endif
 #ifdef HAVE_X11_EXTENSIONS_XF86MISC_H
@@ -374,34 +364,24 @@ apply_settings (GConfClient        *client,
                                 KBKeyClickPercent | KBBellPercent | KBBellPitch | KBBellDuration,
                                 &kbdcontrol);
 
-
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
-        if (!numlock_setup_error && rnumlock) {
-                GConfClient *client;
-                client = gconf_client_get_default ();
+        if (manager->priv->have_xkb && rnumlock) {
                 numlock_set_xkb_state (numlock_get_gconf_state (client));
-                g_object_unref (client);
         }
-        numlock_starting_up = FALSE;
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         XSync (GDK_DISPLAY (), FALSE);
         gdk_error_trap_pop ();
 }
 
-static void
+static guint
 register_config_callback (GsdKeyboardManager      *manager,
+                          GConfClient             *client,
                           const char              *path,
                           GConfClientNotifyFunc    func)
 {
-        GConfClient *client;
-
-        client = gconf_client_get_default ();
-
         gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
-        gconf_client_notify_add (client, path, func, manager, NULL, NULL);
-
-        g_object_unref (client);
+        return gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
 gboolean
@@ -410,29 +390,64 @@ gsd_keyboard_manager_start (GsdKeyboardManager *manager,
 {
         GConfClient *client;
 
-        client = gconf_client_get_default ();
+        gnome_settings_profile_start (NULL);
+
         g_debug ("Starting keyboard manager");
+
+        manager->priv->have_xkb = 0;
+        client = gconf_client_get_default ();
+
         /* Essential - xkb initialization should happen before */
         gsd_keyboard_xkb_set_post_activation_callback ((PostActivationCallback) gsd_load_modmap_files, NULL);
         gsd_keyboard_xkb_init (client);
-        gsd_keyboard_xkb_load (client);
 
-        register_config_callback (manager,
-                                  GSD_KEYBOARD_KEY,
-                                  (GConfClientNotifyFunc)apply_settings);
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
-        numlock_install_xkb_callback ();
+        numlock_xkb_init (manager);
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
+        /* apply current settings before we install the callback */
+        apply_settings (client, 0, NULL, manager);
+
+        manager->priv->notify = register_config_callback (manager,
+                                                          client,
+                                                          GSD_KEYBOARD_KEY,
+                                                          (GConfClientNotifyFunc) apply_settings);
+
         g_object_unref (client);
 
+#ifdef HAVE_X11_EXTENSIONS_XKB_H
+        numlock_install_xkb_callback (manager);
+#endif /* HAVE_X11_EXTENSIONS_XKB_H */
+
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
 void
 gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
 {
+        GsdKeyboardManagerPrivate *p = manager->priv;
+
         g_debug ("Stopping keyboard manager");
+
+        if (p->notify != 0) {
+                GConfClient *client = gconf_client_get_default ();
+                gconf_client_remove_dir (client, GSD_KEYBOARD_KEY, NULL);
+                gconf_client_notify_remove (client, p->notify);
+                g_object_unref (client);
+                p->notify = 0;
+        }
+
+#if HAVE_X11_EXTENSIONS_XKB_H
+        if (p->have_xkb) {
+                gdk_window_remove_filter (NULL,
+                                          numlock_xkb_callback,
+                                          GINT_TO_POINTER (p->xkb_event_base));
+        }
+#endif /* HAVE_X11_EXTENSIONS_XKB_H */
+
+        gsd_keyboard_xkb_shutdown ();
 }
 
 static void
@@ -514,7 +529,6 @@ static void
 gsd_keyboard_manager_init (GsdKeyboardManager *manager)
 {
         manager->priv = GSD_KEYBOARD_MANAGER_GET_PRIVATE (manager);
-
 }
 
 static void
